@@ -13,6 +13,7 @@ from freqtrade.configuration.deprecated_settings import process_temporary_deprec
 from freqtrade.configuration.directory_operations import create_datadir, create_userdata_dir
 from freqtrade.configuration.environment_vars import enironment_vars_to_dict
 from freqtrade.configuration.load_config import load_file, load_from_files
+from freqtrade.constants import Config
 from freqtrade.enums import NON_UTIL_MODES, TRADING_MODES, CandleType, RunMode, TradingMode
 from freqtrade.exceptions import OperationalException
 from freqtrade.loggers import setup_logging
@@ -30,10 +31,10 @@ class Configuration:
 
     def __init__(self, args: Dict[str, Any], runmode: RunMode = None) -> None:
         self.args = args
-        self.config: Optional[Dict[str, Any]] = None
+        self.config: Optional[Config] = None
         self.runmode = runmode
 
-    def get_config(self) -> Dict[str, Any]:
+    def get_config(self) -> Config:
         """
         Return the config. Use this method to get the bot config
         :return: Dict: Bot config
@@ -65,7 +66,7 @@ class Configuration:
         :return: Configuration dictionary
         """
         # Load all configs
-        config: Dict[str, Any] = load_from_files(self.args.get("config", []))
+        config: Config = load_from_files(self.args.get("config", []))
 
         # Load environment variables
         env_data = enironment_vars_to_dict()
@@ -95,6 +96,10 @@ class Configuration:
 
         self._process_data_options(config)
 
+        self._process_analyze_options(config)
+
+        self._process_freqai_options(config)
+
         # Check if the exchange set by the user is supported
         check_exchange(config, config.get('experimental', {}).get('block_bad_exchanges', True))
 
@@ -104,7 +109,7 @@ class Configuration:
 
         return config
 
-    def _process_logging_options(self, config: Dict[str, Any]) -> None:
+    def _process_logging_options(self, config: Config) -> None:
         """
         Extract information for sys.argv and load logging configuration:
         the -v/--verbose, --logfile options
@@ -117,7 +122,7 @@ class Configuration:
 
         setup_logging(config)
 
-    def _process_trading_options(self, config: Dict[str, Any]) -> None:
+    def _process_trading_options(self, config: Config) -> None:
         if config['runmode'] not in TRADING_MODES:
             return
 
@@ -127,13 +132,13 @@ class Configuration:
                 # Default to in-memory db for dry_run if not specified
                 config['db_url'] = constants.DEFAULT_DB_DRYRUN_URL
         else:
-            if not config.get('db_url', None):
+            if not config.get('db_url'):
                 config['db_url'] = constants.DEFAULT_DB_PROD_URL
             logger.info('Dry run is disabled')
 
         logger.info(f'Using DB: "{parse_db_uri_for_logging(config["db_url"])}"')
 
-    def _process_common_options(self, config: Dict[str, Any]) -> None:
+    def _process_common_options(self, config: Config) -> None:
 
         # Set strategy if not specified in config and or if it's non default
         if self.args.get('strategy') or not config.get('strategy'):
@@ -157,7 +162,7 @@ class Configuration:
         if 'sd_notify' in self.args and self.args['sd_notify']:
             config['internals'].update({'sd_notify': True})
 
-    def _process_datadir_options(self, config: Dict[str, Any]) -> None:
+    def _process_datadir_options(self, config: Config) -> None:
         """
         Extract information for sys.argv and load directory configurations
         --user-data, --datadir
@@ -180,7 +185,7 @@ class Configuration:
         config['user_data_dir'] = create_userdata_dir(config['user_data_dir'], create_dir=False)
         logger.info('Using user-data directory: %s ...', config['user_data_dir'])
 
-        config.update({'datadir': create_datadir(config, self.args.get('datadir', None))})
+        config.update({'datadir': create_datadir(config, self.args.get('datadir'))})
         logger.info('Using data directory: %s ...', config.get('datadir'))
 
         if self.args.get('exportfilename'):
@@ -191,7 +196,7 @@ class Configuration:
             config['exportfilename'] = (config['user_data_dir']
                                         / 'backtest_results')
 
-    def _process_optimize_options(self, config: Dict[str, Any]) -> None:
+    def _process_optimize_options(self, config: Config) -> None:
 
         # This will override the strategy configuration
         self._args_to_config(config, argname='timeframe',
@@ -219,7 +224,7 @@ class Configuration:
         if config.get('max_open_trades') == -1:
             config['max_open_trades'] = float('inf')
 
-        if self.args.get('stake_amount', None):
+        if self.args.get('stake_amount'):
             # Convert explicitly to float to support CLI argument for both unlimited and value
             try:
                 self.args['stake_amount'] = float(self.args['stake_amount'])
@@ -298,6 +303,9 @@ class Configuration:
         self._args_to_config(config, argname='spaces',
                              logstring='Parameter -s/--spaces detected: {}')
 
+        self._args_to_config(config, argname='analyze_per_epoch',
+                             logstring='Parameter --analyze-per-epoch detected.')
+
         self._args_to_config(config, argname='print_all',
                              logstring='Parameter --print-all detected ...')
 
@@ -373,7 +381,7 @@ class Configuration:
         self._args_to_config(config, argname="hyperopt_ignore_missing_space",
                              logstring="Paramter --ignore-missing-space detected: {}")
 
-    def _process_plot_options(self, config: Dict[str, Any]) -> None:
+    def _process_plot_options(self, config: Config) -> None:
 
         self._args_to_config(config, argname='pairs',
                              logstring='Using pairs {}')
@@ -422,7 +430,10 @@ class Configuration:
         self._args_to_config(config, argname='dataformat_trades',
                              logstring='Using "{}" to store trades data.')
 
-    def _process_data_options(self, config: Dict[str, Any]) -> None:
+        self._args_to_config(config, argname='show_timerange',
+                             logstring='Detected --show-timerange')
+
+    def _process_data_options(self, config: Config) -> None:
         self._args_to_config(config, argname='new_pairs_days',
                              logstring='Detected --new-pairs-days: {}')
         self._args_to_config(config, argname='trading_mode',
@@ -433,7 +444,20 @@ class Configuration:
         self._args_to_config(config, argname='candle_types',
                              logstring='Detected --candle-types: {}')
 
-    def _process_runmode(self, config: Dict[str, Any]) -> None:
+    def _process_analyze_options(self, config: Config) -> None:
+        self._args_to_config(config, argname='analysis_groups',
+                             logstring='Analysis reason groups: {}')
+
+        self._args_to_config(config, argname='enter_reason_list',
+                             logstring='Analysis enter tag list: {}')
+
+        self._args_to_config(config, argname='exit_reason_list',
+                             logstring='Analysis exit tag list: {}')
+
+        self._args_to_config(config, argname='indicator_list',
+                             logstring='Analysis indicator list: {}')
+
+    def _process_runmode(self, config: Config) -> None:
 
         self._args_to_config(config, argname='dry_run',
                              logstring='Parameter --dry-run detected, '
@@ -446,7 +470,17 @@ class Configuration:
 
         config.update({'runmode': self.runmode})
 
-    def _args_to_config(self, config: Dict[str, Any], argname: str,
+    def _process_freqai_options(self, config: Config) -> None:
+
+        self._args_to_config(config, argname='freqaimodel',
+                             logstring='Using freqaimodel class name: {}')
+
+        self._args_to_config(config, argname='freqaimodel_path',
+                             logstring='Using freqaimodel path: {}')
+
+        return
+
+    def _args_to_config(self, config: Config, argname: str,
                         logstring: str, logfun: Optional[Callable] = None,
                         deprecated_msg: Optional[str] = None) -> None:
         """
@@ -459,7 +493,7 @@ class Configuration:
                         configuration instead of the content)
         """
         if (argname in self.args and self.args[argname] is not None
-           and self.args[argname] is not False):
+                and self.args[argname] is not False):
 
             config.update({argname: self.args[argname]})
             if logfun:
@@ -469,7 +503,7 @@ class Configuration:
             if deprecated_msg:
                 warnings.warn(f"DEPRECATED: {deprecated_msg}", DeprecationWarning)
 
-    def _resolve_pairs_list(self, config: Dict[str, Any]) -> None:
+    def _resolve_pairs_list(self, config: Config) -> None:
         """
         Helper for download script.
         Takes first found:
