@@ -5,12 +5,12 @@ import itertools
 import logging
 from datetime import datetime, timezone
 from operator import itemgetter
-from typing import Any, Dict, List
+from typing import Dict, List
 
 import pandas as pd
 from pandas import DataFrame, to_datetime
 
-from freqtrade.constants import DEFAULT_DATAFRAME_COLUMNS, DEFAULT_TRADES_COLUMNS, TradeList
+from freqtrade.constants import DEFAULT_DATAFRAME_COLUMNS, DEFAULT_TRADES_COLUMNS, Config, TradeList
 from freqtrade.enums import CandleType
 
 
@@ -237,7 +237,7 @@ def trades_to_ohlcv(trades: TradeList, timeframe: str) -> DataFrame:
     return df_new.loc[:, DEFAULT_DATAFRAME_COLUMNS]
 
 
-def convert_trades_format(config: Dict[str, Any], convert_from: str, convert_to: str, erase: bool):
+def convert_trades_format(config: Config, convert_from: str, convert_to: str, erase: bool):
     """
     Convert trades from one format to another format.
     :param config: Config dictionary
@@ -263,7 +263,7 @@ def convert_trades_format(config: Dict[str, Any], convert_from: str, convert_to:
 
 
 def convert_ohlcv_format(
-    config: Dict[str, Any],
+    config: Config,
     convert_from: str,
     convert_to: str,
     erase: bool,
@@ -292,6 +292,7 @@ def convert_ohlcv_format(
                 timeframe,
                 candle_type=candle_type
             ))
+        config['pairs'] = sorted(set(config['pairs']))
     logger.info(f"Converting candle (OHLCV) data for {config['pairs']}")
 
     for timeframe in timeframes:
@@ -302,7 +303,7 @@ def convert_ohlcv_format(
                                   drop_incomplete=False,
                                   startup_candles=0,
                                   candle_type=candle_type)
-            logger.info(f"Converting {len(data)} {candle_type} candles for {pair}")
+            logger.info(f"Converting {len(data)} {timeframe} {candle_type} candles for {pair}")
             if len(data) > 0:
                 trg.ohlcv_store(
                     pair=pair,
