@@ -5,6 +5,7 @@ import logging
 import random
 from typing import Any, Dict, List
 
+from freqtrade.constants import Config
 from freqtrade.enums import RunMode
 from freqtrade.plugins.pairlist.IPairList import IPairList
 
@@ -15,7 +16,7 @@ logger = logging.getLogger(__name__)
 class ShuffleFilter(IPairList):
 
     def __init__(self, exchange, pairlistmanager,
-                 config: Dict[str, Any], pairlistconfig: Dict[str, Any],
+                 config: Config, pairlistconfig: Dict[str, Any],
                  pairlist_pos: int) -> None:
         super().__init__(exchange, pairlistmanager, config, pairlistconfig, pairlist_pos)
 
