@@ -4,14 +4,14 @@ Volume PairList provider
 Provides dynamic pair list based on trade volumes
 """
 import logging
+from datetime import datetime, timedelta, timezone
 from typing import Any, Dict, List
 
-import arrow
 from cachetools import TTLCache
 
-from freqtrade.constants import ListPairsWithTimeframes
+from freqtrade.constants import Config, ListPairsWithTimeframes
 from freqtrade.exceptions import OperationalException
-from freqtrade.exchange import timeframe_to_minutes
+from freqtrade.exchange import timeframe_to_minutes, timeframe_to_prev_date
 from freqtrade.misc import format_ms_time
 from freqtrade.plugins.pairlist.IPairList import IPairList
 
@@ -25,7 +25,7 @@ SORT_VALUES = ['quoteVolume']
 class VolumePairList(IPairList):
 
     def __init__(self, exchange, pairlistmanager,
-                 config: Dict[str, Any], pairlistconfig: Dict[str, Any],
+                 config: Config, pairlistconfig: Dict[str, Any],
                  pairlist_pos: int) -> None:
         super().__init__(exchange, pairlistmanager, config, pairlistconfig, pairlist_pos)
 
@@ -73,7 +73,7 @@ class VolumePairList(IPairList):
 
         if (not self._use_range and not (
                 self._exchange.exchange_has('fetchTickers')
-                and self._exchange._ft_has["tickers_have_quoteVolume"])):
+                and self._exchange.get_option("tickers_have_quoteVolume"))):
             raise OperationalException(
                 "Exchange does not support dynamic whitelist in this configuration. "
                 "Please edit your config and either remove Volumepairlist, "
@@ -158,16 +158,16 @@ class VolumePairList(IPairList):
             filtered_tickers: List[Dict[str, Any]] = [{'symbol': k} for k in pairlist]
 
             # get lookback period in ms, for exchange ohlcv fetch
-            since_ms = int(arrow.utcnow()
-                           .floor('minute')
-                           .shift(minutes=-(self._lookback_period * self._tf_in_min)
-                                  - self._tf_in_min)
-                           .int_timestamp) * 1000
+            since_ms = int(timeframe_to_prev_date(
+                self._lookback_timeframe,
+                datetime.now(timezone.utc) + timedelta(
+                    minutes=-(self._lookback_period * self._tf_in_min) - self._tf_in_min)
+                    ).timestamp()) * 1000
 
-            to_ms = int(arrow.utcnow()
-                        .floor('minute')
-                        .shift(minutes=-self._tf_in_min)
-                        .int_timestamp) * 1000
+            to_ms = int(timeframe_to_prev_date(
+                            self._lookback_timeframe,
+                            datetime.now(timezone.utc) - timedelta(minutes=self._tf_in_min)
+                            ).timestamp()) * 1000
 
             # todo: utc date output for starting date
             self.log_once(f"Using volume range of {self._lookback_period} candles, timeframe: "
@@ -186,6 +186,7 @@ class VolumePairList(IPairList):
                     needed_pairs, since_ms=since_ms, cache=False
                 )
             for i, p in enumerate(filtered_tickers):
+                contract_size = self._exchange.markets[p['symbol']].get('contractSize', 1.0) or 1.0
                 pair_candles = candles[
                     (p['symbol'], self._lookback_timeframe, self._def_candletype)
                 ] if (
@@ -193,12 +194,13 @@ class VolumePairList(IPairList):
                     ) in candles else None
                 # in case of candle data calculate typical price and quoteVolume for candle
                 if pair_candles is not None and not pair_candles.empty:
-                    if self._exchange._ft_has["ohlcv_volume_currency"] == "base":
+                    if self._exchange.get_option("ohlcv_volume_currency") == "base":
                         pair_candles['typical_price'] = (pair_candles['high'] + pair_candles['low']
                                                          + pair_candles['close']) / 3
 
                         pair_candles['quoteVolume'] = (
                             pair_candles['volume'] * pair_candles['typical_price']
+                            * contract_size
                         )
                     else:
                         # Exchange ohlcv data is in quote volume already.
