@@ -18,9 +18,9 @@ def get_rpc_optional() -> Optional[RPC]:
 def get_rpc() -> Optional[Iterator[RPC]]:
     _rpc = get_rpc_optional()
     if _rpc:
-        Trade.query.session.rollback()
+        Trade.rollback()
         yield _rpc
-        Trade.query.session.rollback()
+        Trade.rollback()
     else:
         raise RPCException('Bot is not in the correct state')
 
@@ -37,10 +37,14 @@ def get_exchange(config=Depends(get_config)):
     if not ApiServer._exchange:
         from freqtrade.resolvers import ExchangeResolver
         ApiServer._exchange = ExchangeResolver.load_exchange(
-            config['exchange']['name'], config)
+            config['exchange']['name'], config, load_leverage_tiers=False)
     return ApiServer._exchange
 
 
+def get_channel_manager():
+    return ApiServer._ws_channel_manager
+
+
 def is_webserver_mode(config=Depends(get_config)):
     if config['runmode'] != RunMode.WEBSERVER:
         raise RPCException('Bot is not in the correct state')
