@@ -7,7 +7,7 @@ from typing import Dict, NamedTuple, Optional
 
 import arrow
 
-from freqtrade.constants import UNLIMITED_STAKE_AMOUNT
+from freqtrade.constants import UNLIMITED_STAKE_AMOUNT, Config
 from freqtrade.enums import RunMode, TradingMode
 from freqtrade.exceptions import DependencyException
 from freqtrade.exchange import Exchange
@@ -35,7 +35,7 @@ class PositionWallet(NamedTuple):
 
 class Wallets:
 
-    def __init__(self, config: dict, exchange: Exchange, log: bool = True) -> None:
+    def __init__(self, config: Config, exchange: Exchange, log: bool = True) -> None:
         self._config = config
         self._log = log
         self._exchange = exchange
@@ -131,9 +131,9 @@ class Wallets:
             if isinstance(balances[currency], dict):
                 self._wallets[currency] = Wallet(
                     currency,
-                    balances[currency].get('free', None),
-                    balances[currency].get('used', None),
-                    balances[currency].get('total', None)
+                    balances[currency].get('free'),
+                    balances[currency].get('used'),
+                    balances[currency].get('total')
                 )
         # Remove currencies no longer in get_balances output
         for currency in deepcopy(self._wallets):
@@ -148,7 +148,7 @@ class Wallets:
                 # Position is not open ...
                 continue
             size = self._exchange._contracts_to_amount(symbol, position['contracts'])
-            collateral = position['collateral']
+            collateral = position['collateral'] or 0.0
             leverage = position['leverage']
             self._positions[symbol] = PositionWallet(
                 symbol, position=size,
