@@ -23,7 +23,7 @@ from tests.exchange.test_exchange import ccxt_exceptionhandlers
 def test_stoploss_order_binance(default_conf, mocker, limitratio, expected, side, trademode):
     api_mock = MagicMock()
     order_id = 'test_prod_buy_{}'.format(randint(0, 10 ** 6))
-    order_type = 'stop_loss_limit' if trademode == TradingMode.SPOT else 'stop'
+    order_type = 'stop_loss_limit' if trademode == TradingMode.SPOT else 'limit'
 
     api_mock.create_order = MagicMock(return_value={
         'id': order_id,
@@ -45,12 +45,15 @@ def test_stoploss_order_binance(default_conf, mocker, limitratio, expected, side
             amount=1,
             stop_price=190,
             side=side,
-            order_types={'stoploss_on_exchange_limit_ratio': 1.05},
+            order_types={'stoploss': 'limit', 'stoploss_on_exchange_limit_ratio': 1.05},
             leverage=1.0
         )
 
     api_mock.create_order.reset_mock()
-    order_types = {} if limitratio is None else {'stoploss_on_exchange_limit_ratio': limitratio}
+    order_types = {'stoploss': 'limit'}
+    if limitratio is not None:
+        order_types.update({'stoploss_on_exchange_limit_ratio': limitratio})
+
     order = exchange.stoploss(
         pair='ETH/BTC',
         amount=1,
@@ -154,6 +157,7 @@ def test_stoploss_adjust_binance(mocker, default_conf, sl1, sl2, sl3, side):
     order = {
         'type': 'stop_loss_limit',
         'price': 1500,
+        'stopPrice': 1500,
         'info': {'stopPrice': 1500},
     }
     assert exchange.stoploss_adjust(sl1, order, side=side)
@@ -375,96 +379,96 @@ def test_fill_leverage_tiers_binance(default_conf, mocker):
     assert exchange._leverage_tiers == {
         'ADA/BUSD': [
             {
-                "min": 0,
-                "max": 100000,
-                "mmr": 0.025,
-                "lev": 20,
+                "minNotional": 0,
+                "maxNotional": 100000,
+                "maintenanceMarginRate": 0.025,
+                "maxLeverage": 20,
                 "maintAmt": 0.0
             },
             {
-                "min": 100000,
-                "max": 500000,
-                "mmr": 0.05,
-                "lev": 10,
+                "minNotional": 100000,
+                "maxNotional": 500000,
+                "maintenanceMarginRate": 0.05,
+                "maxLeverage": 10,
                 "maintAmt": 2500.0
             },
             {
-                "min": 500000,
-                "max": 1000000,
-                "mmr": 0.1,
-                "lev": 5,
+                "minNotional": 500000,
+                "maxNotional": 1000000,
+                "maintenanceMarginRate": 0.1,
+                "maxLeverage": 5,
                 "maintAmt": 27500.0
             },
             {
-                "min": 1000000,
-                "max": 2000000,
-                "mmr": 0.15,
-                "lev": 3,
+                "minNotional": 1000000,
+                "maxNotional": 2000000,
+                "maintenanceMarginRate": 0.15,
+                "maxLeverage": 3,
                 "maintAmt": 77500.0
             },
             {
-                "min": 2000000,
-                "max": 5000000,
-                "mmr": 0.25,
-                "lev": 2,
+                "minNotional": 2000000,
+                "maxNotional": 5000000,
+                "maintenanceMarginRate": 0.25,
+                "maxLeverage": 2,
                 "maintAmt": 277500.0
             },
             {
-                "min": 5000000,
-                "max": 30000000,
-                "mmr": 0.5,
-                "lev": 1,
+                "minNotional": 5000000,
+                "maxNotional": 30000000,
+                "maintenanceMarginRate": 0.5,
+                "maxLeverage": 1,
                 "maintAmt": 1527500.0
             }
         ],
         "ZEC/USDT": [
             {
-                'min': 0,
-                'max': 50000,
-                'mmr': 0.01,
-                'lev': 50,
+                'minNotional': 0,
+                'maxNotional': 50000,
+                'maintenanceMarginRate': 0.01,
+                'maxLeverage': 50,
                 'maintAmt': 0.0
             },
             {
-                'min': 50000,
-                'max': 150000,
-                'mmr': 0.025,
-                'lev': 20,
+                'minNotional': 50000,
+                'maxNotional': 150000,
+                'maintenanceMarginRate': 0.025,
+                'maxLeverage': 20,
                 'maintAmt': 750.0
             },
             {
-                'min': 150000,
-                'max': 250000,
-                'mmr': 0.05,
-                'lev': 10,
+                'minNotional': 150000,
+                'maxNotional': 250000,
+                'maintenanceMarginRate': 0.05,
+                'maxLeverage': 10,
                 'maintAmt': 4500.0
             },
             {
-                'min': 250000,
-                'max': 500000,
-                'mmr': 0.1,
-                'lev': 5,
+                'minNotional': 250000,
+                'maxNotional': 500000,
+                'maintenanceMarginRate': 0.1,
+                'maxLeverage': 5,
                 'maintAmt': 17000.0
             },
             {
-                'min': 500000,
-                'max': 1000000,
-                'mmr': 0.125,
-                'lev': 4,
+                'minNotional': 500000,
+                'maxNotional': 1000000,
+                'maintenanceMarginRate': 0.125,
+                'maxLeverage': 4,
                 'maintAmt': 29500.0
             },
             {
-                'min': 1000000,
-                'max': 2000000,
-                'mmr': 0.25,
-                'lev': 2,
+                'minNotional': 1000000,
+                'maxNotional': 2000000,
+                'maintenanceMarginRate': 0.25,
+                'maxLeverage': 2,
                 'maintAmt': 154500.0
             },
             {
-                'min': 2000000,
-                'max': 30000000,
-                'mmr': 0.5,
-                'lev': 1,
+                'minNotional': 2000000,
+                'maxNotional': 30000000,
+                'maintenanceMarginRate': 0.5,
+                'maxLeverage': 1,
                 'maintAmt': 654500.0
             },
         ]
