@@ -137,7 +137,12 @@ def exchange_futures(request, exchange_conf, class_mocker):
             'freqtrade.exchange.binance.Binance.fill_leverage_tiers')
         class_mocker.patch('freqtrade.exchange.exchange.Exchange.fetch_trading_fees')
         class_mocker.patch('freqtrade.exchange.okx.Okx.additional_exchange_init')
-        exchange = ExchangeResolver.load_exchange(request.param, exchange_conf, validate=True)
+        class_mocker.patch('freqtrade.exchange.exchange.Exchange.load_cached_leverage_tiers',
+                           return_value=None)
+        class_mocker.patch('freqtrade.exchange.exchange.Exchange.cache_leverage_tiers')
+
+        exchange = ExchangeResolver.load_exchange(
+            request.param, exchange_conf, validate=True, load_leverage_tiers=True)
 
         yield exchange, request.param
 
@@ -153,6 +158,25 @@ class TestCCXTExchange():
         assert isinstance(markets[pair], dict)
         assert exchange.market_is_spot(markets[pair])
 
+    def test_has_validations(self, exchange):
+
+        exchange, exchangename = exchange
+
+        exchange.validate_ordertypes({
+            'entry': 'limit',
+            'exit': 'limit',
+            'stoploss': 'limit',
+            })
+
+        if exchangename == 'gateio':
+            # gateio doesn't have market orders on spot
+            return
+        exchange.validate_ordertypes({
+            'entry': 'market',
+            'exit': 'market',
+            'stoploss': 'market',
+            })
+
     def test_load_markets_futures(self, exchange_futures):
         exchange, exchangename = exchange_futures
         if not exchange:
@@ -199,8 +223,13 @@ class TestCCXTExchange():
         l2 = exchange.fetch_l2_order_book(pair)
         assert 'asks' in l2
         assert 'bids' in l2
+        assert len(l2['asks']) >= 1
+        assert len(l2['bids']) >= 1
         l2_limit_range = exchange._ft_has['l2_limit_range']
         l2_limit_range_required = exchange._ft_has['l2_limit_range_required']
+        if exchangename == 'gateio':
+            # TODO: Gateio is unstable here at the moment, ignoring the limit partially.
+            return
         for val in [1, 2, 5, 25, 100]:
             l2 = exchange.fetch_l2_order_book(pair, val)
             if not l2_limit_range or val in l2_limit_range:
@@ -238,13 +267,8 @@ class TestCCXTExchange():
         now = datetime.now(timezone.utc) - timedelta(minutes=(timeframe_to_minutes(timeframe) * 2))
         assert exchange.klines(pair_tf).iloc[-1]['date'] >= timeframe_to_prev_date(timeframe, now)
 
-    def test_ccxt__async_get_candle_history(self, exchange):
-        exchange, exchangename = exchange
-        # For some weired reason, this test returns random lengths for bittrex.
-        if not exchange._ft_has['ohlcv_has_history'] or exchangename == 'bittrex':
-            return
-        pair = EXCHANGES[exchangename]['pair']
-        timeframe = EXCHANGES[exchangename]['timeframe']
+    def ccxt__async_get_candle_history(self, exchange, exchangename, pair, timeframe):
+
         candle_type = CandleType.SPOT
         timeframe_ms = timeframe_to_msecs(timeframe)
         now = timeframe_to_prev_date(
@@ -270,6 +294,24 @@ class TestCCXTExchange():
             assert len(candles) >= min(candle_count, candle_count1)
             assert candles[0][0] == since_ms or (since_ms + timeframe_ms)
 
+    def test_ccxt__async_get_candle_history(self, exchange):
+        exchange, exchangename = exchange
+        # For some weired reason, this test returns random lengths for bittrex.
+        if not exchange._ft_has['ohlcv_has_history'] or exchangename in ('bittrex'):
+            return
+        pair = EXCHANGES[exchangename]['pair']
+        timeframe = EXCHANGES[exchangename]['timeframe']
+        self.ccxt__async_get_candle_history(exchange, exchangename, pair, timeframe)
+
+    def test_ccxt__async_get_candle_history_futures(self, exchange_futures):
+        exchange, exchangename = exchange_futures
+        if not exchange:
+            # exchange_futures only returns values for supported exchanges
+            return
+        pair = EXCHANGES[exchangename].get('futures_pair', EXCHANGES[exchangename]['pair'])
+        timeframe = EXCHANGES[exchangename]['timeframe']
+        self.ccxt__async_get_candle_history(exchange, exchangename, pair, timeframe)
+
     def test_ccxt_fetch_funding_rate_history(self, exchange_futures):
         exchange, exchangename = exchange_futures
         if not exchange:
@@ -380,14 +422,14 @@ class TestCCXTExchange():
                 assert (isinstance(futures_leverage, float) or isinstance(futures_leverage, int))
                 assert futures_leverage >= 1.0
 
-    def test_ccxt__get_contract_size(self, exchange_futures):
+    def test_ccxt_get_contract_size(self, exchange_futures):
         futures, futures_name = exchange_futures
         if futures:
             futures_pair = EXCHANGES[futures_name].get(
                 'futures_pair',
                 EXCHANGES[futures_name]['pair']
             )
-            contract_size = futures._get_contract_size(futures_pair)
+            contract_size = futures.get_contract_size(futures_pair)
             assert (isinstance(contract_size, float) or isinstance(contract_size, int))
             assert contract_size >= 0.0
 
@@ -439,6 +481,7 @@ class TestCCXTExchange():
                 False,
                 100,
                 100,
+                100,
             )
             assert (isinstance(liquidation_price, float))
             assert liquidation_price >= 0.0
@@ -449,6 +492,7 @@ class TestCCXTExchange():
                 False,
                 100,
                 100,
+                100,
             )
             assert (isinstance(liquidation_price, float))
             assert liquidation_price >= 0.0
