@@ -52,13 +52,6 @@ def trim_dictlist(dict_list, num):
     return new
 
 
-@pytest.fixture(autouse=True)
-def backtesting_cleanup() -> None:
-    yield None
-
-    Backtesting.cleanup()
-
-
 def load_data_test(what, testdatadir):
     timerange = TimeRange.parse_timerange('1510694220-1510700340')
     data = history.load_pair_history(pair='UNITTEST/BTC', datadir=testdatadir,
@@ -90,28 +83,6 @@ def load_data_test(what, testdatadir):
                                                   fill_missing=True)}
 
 
-def simple_backtest(config, contour, mocker, testdatadir) -> None:
-    patch_exchange(mocker)
-    config['timeframe'] = '1m'
-    backtesting = Backtesting(config)
-    backtesting._set_strategy(backtesting.strategylist[0])
-
-    data = load_data_test(contour, testdatadir)
-    processed = backtesting.strategy.advise_all_indicators(data)
-    min_date, max_date = get_timerange(processed)
-    assert isinstance(processed, dict)
-    results = backtesting.backtest(
-        processed=processed,
-        start_date=min_date,
-        end_date=max_date,
-        max_open_trades=1,
-        position_stacking=False,
-        enable_protections=config.get('enable_protections', False),
-    )
-    # results :: <class 'pandas.core.frame.DataFrame'>
-    return results
-
-
 # FIX: fixturize this?
 def _make_backtest_conf(mocker, datadir, conf=None, pair='UNITTEST/BTC'):
     data = history.load_data(datadir=datadir, timeframe='1m', pairs=[pair])
@@ -456,7 +427,7 @@ def test_backtesting_no_pair_left(default_conf, mocker, caplog, testdatadir) ->
 
     default_conf['pairlists'] = [{"method": "VolumePairList", "number_assets": 5}]
     with pytest.raises(OperationalException,
-                       match=r'VolumePairList not allowed for backtesting\..*StaticPairlist.*'):
+                       match=r'VolumePairList not allowed for backtesting\..*StaticPairList.*'):
         Backtesting(default_conf)
 
     default_conf.update({
@@ -489,7 +460,7 @@ def test_backtesting_pairlist_list(default_conf, mocker, caplog, testdatadir, ti
 
     default_conf['pairlists'] = [{"method": "VolumePairList", "number_assets": 5}]
     with pytest.raises(OperationalException,
-                       match=r'VolumePairList not allowed for backtesting\..*StaticPairlist.*'):
+                       match=r'VolumePairList not allowed for backtesting\..*StaticPairList.*'):
         Backtesting(default_conf)
 
     default_conf['pairlists'] = [{"method": "StaticPairList"}, {"method": "PerformanceFilter"}]
@@ -572,6 +543,7 @@ def test_backtest__enter_trade_futures(default_conf_usdt, fee, mocker) -> None:
     mocker.patch("freqtrade.exchange.Exchange.get_min_pair_stake_amount", return_value=0.00001)
     mocker.patch("freqtrade.exchange.Exchange.get_max_pair_stake_amount", return_value=float('inf'))
     mocker.patch("freqtrade.exchange.Exchange.get_max_leverage", return_value=100)
+    mocker.patch("freqtrade.optimize.backtesting.price_to_precision", lambda p, *args: p)
     patch_exchange(mocker)
     default_conf_usdt['stake_amount'] = 300
     default_conf_usdt['max_open_trades'] = 2
@@ -581,13 +553,13 @@ def test_backtest__enter_trade_futures(default_conf_usdt, fee, mocker) -> None:
     default_conf_usdt['exchange']['pair_whitelist'] = ['.*']
     backtesting = Backtesting(default_conf_usdt)
     backtesting._set_strategy(backtesting.strategylist[0])
-    pair = 'UNITTEST/USDT:USDT'
+    pair = 'ETH/USDT:USDT'
     row = [
         pd.Timestamp(year=2020, month=1, day=1, hour=5, minute=0),
-        0.001,  # Open
-        0.0012,  # High
-        0.00099,  # Low
-        0.0011,  # Close
+        0.1,  # Open
+        0.12,  # High
+        0.099,  # Low
+        0.11,  # Close
         1,  # enter_long
         0,  # exit_long
         1,  # enter_short
@@ -602,8 +574,8 @@ def test_backtest__enter_trade_futures(default_conf_usdt, fee, mocker) -> None:
                  return_value=(0.01, 0.01))
 
     # leverage = 5
-    # ep1(trade.open_rate) = 0.001
-    # position(trade.amount) = 1500000
+    # ep1(trade.open_rate) = 0.1
+    # position(trade.amount) = 15000
     # stake_amount = 300 -> wb = 300 / 5 = 60
     # mmr = 0.01
     # cum_b = 0.01
@@ -613,26 +585,26 @@ def test_backtest__enter_trade_futures(default_conf_usdt, fee, mocker) -> None:
     # Binance, Long
     # liquidation_price
     #   = ((wb + cum_b) - (side_1 * position * ep1)) / ((position * mmr_b) - (side_1 * position))
-    #   = ((300 + 0.01) - (1 * 1500000 * 0.001)) / ((1500000 * 0.01) - (1 * 1500000))
+    #   = ((300 + 0.01) - (1 * 15000 * 0.1)) / ((15000 * 0.01) - (1 * 15000))
     #   = 0.0008080740740740741
     # freqtrade_liquidation_price = liq + (abs(open_rate - liq) * liq_buffer * side_1)
-    #   = 0.0008080740740740741 + ((0.001 - 0.0008080740740740741) * 0.05 * 1)
-    #   = 0.0008176703703703704
+    #   = 0.08080740740740741 + ((0.1 - 0.08080740740740741) * 0.05 * 1)
+    #   = 0.08176703703703704
 
     trade = backtesting._enter_trade(pair, row=row, direction='long')
-    assert pytest.approx(trade.liquidation_price) == 0.00081767037
+    assert pytest.approx(trade.liquidation_price) == 0.081767037
 
     # Binance, Short
     # liquidation_price
     #   = ((wb + cum_b) - (side_1 * position * ep1)) / ((position * mmr_b) - (side_1 * position))
-    #   = ((300 + 0.01) - ((-1) * 1500000 * 0.001)) / ((1500000 * 0.01) - ((-1) * 1500000))
+    #   = ((300 + 0.01) - ((-1) * 15000 * 0.1)) / ((15000 * 0.01) - ((-1) * 15000))
     #   = 0.0011881254125412541
     # freqtrade_liquidation_price = liq + (abs(open_rate - liq) * liq_buffer * side_1)
-    #   = 0.0011881254125412541 + (abs(0.001 - 0.0011881254125412541) * 0.05 * -1)
-    #   = 0.0011787191419141915
+    #   = 0.11881254125412541 + (abs(0.1 - 0.11881254125412541) * 0.05 * -1)
+    #   = 0.11787191419141915
 
     trade = backtesting._enter_trade(pair, row=row, direction='short')
-    assert pytest.approx(trade.liquidation_price) == 0.0011787191
+    assert pytest.approx(trade.liquidation_price) == 0.11787191
 
     # Stake-amount too high!
     mocker.patch("freqtrade.exchange.Exchange.get_min_pair_stake_amount", return_value=600.0)
@@ -795,10 +767,27 @@ def test_backtest_one(default_conf, fee, mocker, testdatadir) -> None:
          'is_open': [False, False],
          'enter_tag': [None, None],
          "is_short": [False, False],
+         'open_timestamp': [1517251200000, 1517283000000],
+         'close_timestamp': [1517265300000, 1517285400000],
+         'orders': [
+            [
+                {'amount': 0.00957442, 'safe_price': 0.104445, 'ft_order_side': 'buy',
+                 'order_filled_timestamp': 1517251200000, 'ft_is_entry': True},
+                {'amount': 0.00957442, 'safe_price': 0.10496853383458644, 'ft_order_side': 'sell',
+                 'order_filled_timestamp': 1517265300000, 'ft_is_entry': False}
+            ], [
+                {'amount': 0.0097064, 'safe_price': 0.10302485, 'ft_order_side': 'buy',
+                 'order_filled_timestamp': 1517283000000, 'ft_is_entry': True},
+                {'amount': 0.0097064, 'safe_price': 0.10354126528822055, 'ft_order_side': 'sell',
+                 'order_filled_timestamp': 1517285400000, 'ft_is_entry': False}
+            ]
+         ]
          })
     pd.testing.assert_frame_equal(results, expected)
+    assert 'orders' in results.columns
     data_pair = processed[pair]
     for _, t in results.iterrows():
+        assert len(t['orders']) == 2
         ln = data_pair.loc[data_pair["date"] == t["open_date"]]
         # Check open trade rate alignes to open rate
         assert ln is not None
@@ -850,7 +839,7 @@ def test_backtest_trim_no_data_left(default_conf, fee, mocker, testdatadir) -> N
     data = history.load_data(datadir=testdatadir, timeframe='5m', pairs=['UNITTEST/BTC'],
                              timerange=timerange)
     df = data['UNITTEST/BTC']
-    df.loc[:, 'date'] = df.loc[:, 'date'] - timedelta(days=1)
+    df['date'] = df.loc[:, 'date'] - timedelta(days=1)
     # Trimming 100 candles, so after 2nd trimming, no candle is left.
     df = df.iloc[:100]
     data['XRP/USDT'] = df
@@ -925,6 +914,7 @@ def test_backtest_dataprovider_analyzed_df(default_conf, fee, mocker, testdatadi
 def test_backtest_pricecontours_protections(default_conf, fee, mocker, testdatadir) -> None:
     # While this test IS a copy of test_backtest_pricecontours, it's needed to ensure
     # results do not carry-over to the next run, which is not given by using parametrize.
+    patch_exchange(mocker)
     default_conf['protections'] = [
         {
             "method": "CooldownPeriod",
@@ -932,6 +922,7 @@ def test_backtest_pricecontours_protections(default_conf, fee, mocker, testdatad
         }]
 
     default_conf['enable_protections'] = True
+    default_conf['timeframe'] = '1m'
     mocker.patch('freqtrade.exchange.Exchange.get_fee', fee)
     mocker.patch("freqtrade.exchange.Exchange.get_min_pair_stake_amount", return_value=0.00001)
     mocker.patch("freqtrade.exchange.Exchange.get_max_pair_stake_amount", return_value=float('inf'))
@@ -942,12 +933,27 @@ def test_backtest_pricecontours_protections(default_conf, fee, mocker, testdatad
         ['sine', 9],
         ['raise', 10],
     ]
+    backtesting = Backtesting(default_conf)
+    backtesting._set_strategy(backtesting.strategylist[0])
+
     # While entry-signals are unrealistic, running backtesting
     # over and over again should not cause different results
     for [contour, numres] in tests:
         # Debug output for random test failure
         print(f"{contour}, {numres}")
-        assert len(simple_backtest(default_conf, contour, mocker, testdatadir)['results']) == numres
+        data = load_data_test(contour, testdatadir)
+        processed = backtesting.strategy.advise_all_indicators(data)
+        min_date, max_date = get_timerange(processed)
+        assert isinstance(processed, dict)
+        results = backtesting.backtest(
+            processed=processed,
+            start_date=min_date,
+            end_date=max_date,
+            max_open_trades=1,
+            position_stacking=False,
+            enable_protections=default_conf.get('enable_protections', False),
+        )
+        assert len(results['results']) == numres
 
 
 @pytest.mark.parametrize('protections,contour,expected', [
@@ -973,7 +979,25 @@ def test_backtest_pricecontours(default_conf, fee, mocker, testdatadir,
     mocker.patch('freqtrade.exchange.Exchange.get_fee', fee)
     # While entry-signals are unrealistic, running backtesting
     # over and over again should not cause different results
-    assert len(simple_backtest(default_conf, contour, mocker, testdatadir)['results']) == expected
+
+    patch_exchange(mocker)
+    default_conf['timeframe'] = '1m'
+    backtesting = Backtesting(default_conf)
+    backtesting._set_strategy(backtesting.strategylist[0])
+
+    data = load_data_test(contour, testdatadir)
+    processed = backtesting.strategy.advise_all_indicators(data)
+    min_date, max_date = get_timerange(processed)
+    assert isinstance(processed, dict)
+    results = backtesting.backtest(
+        processed=processed,
+        start_date=min_date,
+        end_date=max_date,
+        max_open_trades=1,
+        position_stacking=False,
+        enable_protections=default_conf.get('enable_protections', False),
+    )
+    assert len(results['results']) == expected
 
 
 def test_backtest_clash_buy_sell(mocker, default_conf, testdatadir):
